package conferenciaBDAlunos;
import java.util.*;

public class Artigo {
  
    private String nome;
    private String resumo;
    private int identificador;
    private String nomeArquivo;
    private double notaFinal;
    private int status;
    private boolean temRevisao;

    private List<Autor> autores;
    private List<Avaliador> avaliadoresConflitantes; //tem conflito com avaliadores
    private List<Topico> topicos;
    private List<Revisao> revisoes;
	    
    
    public Artigo (String nome){
    	this.nome = nome;
    	this.status = 0; //por default e' nao aceito
    	this.autores = new ArrayList<Autor> ();
    	this.avaliadoresConflitantes = new ArrayList<Avaliador> ();
    	this.revisoes = new ArrayList<Revisao> ();
    	this.topicos = new ArrayList<Topico> ();  	
    	this.temRevisao = false;
    }

	public int getIdentificador() {
		return identificador;
	}

	public String getNome() {
		return nome;
	}

	public String getNomeArquivo() {
		return nomeArquivo;
	}

	public double getNotaFinal() {
		return notaFinal;
	}

	public String getResumo() {
		return resumo;
	}

	public int getStatus() {
		return status;
	}

	public void setIdentificador(int identificador) {
		this.identificador = identificador;
	}

	public void setNome(String nome) {
		this.nome = nome;
	}

	public void setNomeArquivo(String nomeArquivo) {
		this.nomeArquivo = nomeArquivo;
	}

	public void setNotaFinal(double notaFinal) {
		this.notaFinal = notaFinal;
	}

	public void setResumo(String resumo) {
		this.resumo = resumo;
	}

	public void setStatus(int status) {
		this.status = status;
	}

	public void setTemRevisao(boolean temRevisao) {
		this.temRevisao = temRevisao;
	}

    public void associaAutor(Autor autor){
    	if (!this.autores.contains(autor))
	    	this.autores.add(autor);
    }

    public void associaTopico(Topico topico){
    	if (!this.topicos.contains(topico))
	    	this.topicos.add(topico);
    }

    public void defineConflito(Avaliador avaliador){
    	if (!this.avaliadoresConflitantes.contains(avaliador))
	    	this.avaliadoresConflitantes.add(avaliador);
    }

    public void associaRevisao(Revisao revisao){
    	this.temRevisao = true;
    	if (this.revisoes.size() < 3 && !this.revisoes.contains(revisao))
    		this.revisoes.add(revisao);
    }

    public void calculaNotaFinal(){
    	double soma = 0;
    	for (int i=0; i<this.revisoes.size(); i++)
    		soma = soma + (this.revisoes.get(i)).getNotaFinal();
    	this.notaFinal = soma / this.revisoes.size();
    }

    public void defineComoAceito(){
    	this.status = 1;
    }

    public void modificaStatus(){
    	if (this.status == 1)
    		this.status = 0;
    	else
    		this.status = 1;	
    }
    
    public boolean isTemRevisao (){
    	return temRevisao;
    }
    
    public List<Revisao> getRevisoes(){
    	return this.revisoes;
    }
    
    public List<Autor> getAutores(){
    	return this.autores;
    }
    
    public List<Topico> getTopicos(){
    	return this.topicos;
    }
    
    public List<Avaliador> getRevisoresComConflito (){
    	return this.avaliadoresConflitantes;
    }
    
    public boolean equals (Object obj){
    	if (obj instanceof Artigo)
        	return this.nome.equals(((Artigo)obj).nome);    		
    	else
    		return false;
    }

	public String toString (){
		return this.nome + " " + this.identificador + " " + this.status;
	}


}
