package conferenciaBDAlunos;
import java.sql.*;
import java.util.*;

public class MapeadorArtigo {


	private static MapeadorArtigo mapeador;

	private Map <String,Artigo> cache; 

	private MapeadorArtigo() {
		super();
		this.cache = new HashMap<String,Artigo>();
	}

	public static MapeadorArtigo getInstance (){
		if (mapeador == null){
			mapeador = new MapeadorArtigo();
		}	
		return mapeador;
	}

	public Artigo get (Connection con, String nome) throws SQLException{

		//COMPLETAR...
	
	}
		
	
	private void getTopicos(Connection con, Artigo art) throws SQLException{
		PreparedStatement stmt = con.prepareStatement("SELECT NOME_TOPICO FROM TOPICO_ARTIGO " +
			"WHERE NOME_ARTIGO = ?");
		stmt.setString(1,art.getNome());	
		ResultSet rs = stmt.executeQuery();
		try{
			while (rs.next()){
				Topico top = MapeadorTopico.getInstance().get(con, rs.getString("NOME_TOPICO"));
				art.associaTopico(top);
			}	
		}
		finally {
			rs.close();
			stmt.close();		
		}
	
	}


	private void getAutores(Connection con, Artigo art) throws SQLException{
		PreparedStatement stmt = con.prepareStatement("SELECT NOME_AUTOR FROM ARTIGO_AUTOR  " +
			"WHERE NOME_ARTIGO = ?");
		stmt.setString(1,art.getNome());	
		ResultSet rs = stmt.executeQuery();
		try{
			while (rs.next()){
				Autor aut = MapeadorAutor.getInstance().get(con, rs.getString("NOME_AUTOR"));
				art.associaAutor(aut);
			}	
		}
		finally {
			rs.close();
			stmt.close();		
		}
	
	}


	private void getAvaliadoresConflitantes(Connection con, Artigo art) throws SQLException{
		PreparedStatement stmt = con.prepareStatement("SELECT NOME_AVALIADOR FROM ARTIGO_AVALIADORCONFLITANTE   " +
			"WHERE NOME_ARTIGO = ?");
		stmt.setString(1,art.getNome());	
		ResultSet rs = stmt.executeQuery();
		try{
			while (rs.next()){
				Avaliador aval = MapeadorAvaliador.getInstance().get(con,rs.getString("NOME_AVALIADOR"));
				art.defineConflito(aval);
			}	
		}
		finally {
			rs.close();
			stmt.close();		
		}
	
	}

	private void getRevisoes(Connection con, Artigo art) throws SQLException{
		PreparedStatement stmt = con.prepareStatement("SELECT NOME_ARTIGO,NOME_AVALIADOR FROM REVISAO    " +
			"WHERE NOME_ARTIGO = ?");
		stmt.setString(1,art.getNome());	
		ResultSet rs = stmt.executeQuery();
		try{
			while (rs.next()){
				Revisao rev = MapeadorRevisao.getInstance().get(con,rs.getString("NOME_ARTIGO"),rs.getString("NOME_AVALIADOR"));
				art.associaRevisao(rev);
			}	
		}
		finally {
			rs.close();
			stmt.close();		
		}
	
	}

	public void put (Connection con, Artigo art) throws SQLException{
		if (this.get(con,art.getNome())!=null) {
			PreparedStatement stmt = con.prepareStatement("UPDATE ARTIGO SET RESUMO=?,IDENTIFICADOR=?,"+
				"NOME_ARQUIVO=?,NOTA_FINAL=?,STATUS=?,TEM_REVISAO=? WHERE NOME=?");
			try{	
				stmt.setString(1,art.getResumo());	
				stmt.setInt(2,art.getIdentificador());	
				stmt.setString(3,art.getNomeArquivo());
				stmt.setDouble(4,art.getNotaFinal());
				stmt.setInt(5,art.getStatus());
				stmt.setBoolean(6,art.isTemRevisao());	
				stmt.setString(7,art.getNome());	
				int updtCnt = stmt.executeUpdate();
				if (updtCnt == 0) 
					throw new SQLException("Nome inexistente!");
			}
			finally{
				stmt.close();
			}	
			this.putAutoresAvaliadoresConflitantesTopicos(con,art);
		} 
		else {
			PreparedStatement stmt = con.prepareStatement("INSERT INTO ARTIGO (NOME,RESUMO,IDENTIFICADOR,"+
				"NOME_ARQUIVO,NOTA_FINAL,STATUS,TEM_REVISAO) VALUES (?,?,?,?,?,?,?)");
			try{	
				stmt.setString(1,art.getNome());	
				stmt.setString(2,art.getResumo());	
				stmt.setInt(3,art.getIdentificador());	
				stmt.setString(4,art.getNomeArquivo());
				stmt.setDouble(5,art.getNotaFinal());
				stmt.setInt(6,art.getStatus());
				stmt.setBoolean(7,art.isTemRevisao());
				stmt.execute();
			}
			finally {	
				stmt.close();
			}	
			this.putAutoresAvaliadoresConflitantesTopicos(con,art);
		}
		this.cache.put(art.getNome(),art);	
	}
	
	
	private void putAutoresAvaliadoresConflitantesTopicos(Connection con, Artigo art)throws SQLException{
	//atualiza as associacoes com os autores na base de dados
		PreparedStatement stmt = con.prepareStatement("DELETE FROM ARTIGO_AUTOR WHERE NOME_ARTIGO=?");
		try{	
			stmt.setString(1,art.getNome());	
			stmt.executeUpdate();
			List<Autor> autores = art.getAutores();
			for (int i=0; i<autores.size();i++){
				Autor autor = autores.get(i);
				stmt = con.prepareStatement("INSERT INTO ARTIGO_AUTOR (NOME_ARTIGO,NOME_AUTOR) "+
					"VALUES(?,?)");
				stmt.setString(1,art.getNome());	
				stmt.setString(2,autor.getNome());	
				stmt.execute();
			}
		}
		finally{
			stmt.close();
		}
		
	//atualiza as associacoes com os avaliadores conflitantes na base de dados
		stmt = con.prepareStatement("DELETE FROM ARTIGO_AVALIADORCONFLITANTE WHERE NOME_ARTIGO=?");
		try{	
			stmt.setString(1,art.getNome());	
			stmt.executeUpdate();
			List<Avaliador> avaliadores = art.getRevisoresComConflito();
			for (int i=0; i<avaliadores.size();i++){
				Avaliador aval = avaliadores.get(i);
				stmt = con.prepareStatement("INSERT INTO ARTIGO_AVALIADORCONFLITANTE (NOME_ARTIGO,NOME_AVALIADOR) "+
					"VALUES(?,?)");
				stmt.setString(1,art.getNome());	
				stmt.setString(2,aval.getNome());	
				stmt.execute();
			}
		}
		finally {
			stmt.close();		
		}
	
	//atualiza as associacoes com os topicos na base de dados
		stmt = con.prepareStatement("DELETE FROM TOPICO_ARTIGO WHERE NOME_ARTIGO=?");
		try{	
			stmt.setString(1,art.getNome());
			stmt.executeUpdate();
			List<Topico> topicos = art.getTopicos();
			for (int i=0; i<topicos.size();i++){
				Topico topico = topicos.get(i);
				stmt = con.prepareStatement("INSERT INTO TOPICO_ARTIGO (NOME_ARTIGO,NOME_TOPICO) " +
					"VALUES (?,?)");
			stmt.setString(1,art.getNome());
			stmt.setString(2,topico.getNome());		
			}
		}
		finally{
			stmt.close();
		}	
	}


	public List<Artigo> selectArtigosSemRevisao(Connection con) throws SQLException{
		PreparedStatement stmt = con.prepareStatement("SELECT NOME,RESUMO,IDENTIFICADOR,NOME_ARQUIVO,"+
				"NOTA_FINAL,STATUS FROM ARTIGO WHERE TEM_REVISAO = ?");
		stmt.setBoolean(1,false);	
		ResultSet rs = stmt.executeQuery();
		List<Artigo> artigos = new ArrayList<Artigo>();
		try{
			while (rs.next()){
				Artigo art= new Artigo (rs.getString("NOME"));
				art.setResumo(rs.getString("RESUMO"));
				art.setIdentificador(rs.getShort("IDENTIFICADOR"));
				art.setNomeArquivo(rs.getString("NOME_ARQUIVO"));
				art.setNotaFinal(rs.getDouble("NOTA_FINAL"));
				art.setStatus(rs.getShort("STATUS"));
				art.setTemRevisao(false);
				this.cache.put(art.getNome(),art);
				this.getTopicos (con,art);
				this.getAutores (con,art);
				this.getAvaliadoresConflitantes (con,art);
				this.getRevisoes (con,art);			
				artigos.add(art);
			}	
		}
		finally {
			rs.close();
			stmt.close();		
		}
		return artigos;
	}
	
	public List<Artigo> selectTodos(Connection con) throws SQLException{		

		//COMPLETAR...
		
	}
	
}

